
#install.packages('survival')
library(survival)
library(survminer)

#setwd("C:\\Users\\Administrator\\Desktop\\rectum_9_25\\22multiROC")                        #设置工作目录
rt=read.table("2Subtype_time_cli_treated.txt",header=T,sep="\t",check.names=F,row.names=1)
colnames(rt)
#rt <- rt[,-c(3:6,8:10,14)]   
colnames(rt)

#单因素独立预后分析
uniTab=data.frame()
for(i in colnames(rt[,3:ncol(rt)])){
  cox <- coxph(Surv(futime, fustat) ~ rt[,i], data = rt)  #单因素是每次输出一个
  coxSummary = summary(cox)
  uniTab=rbind(uniTab,
               cbind(id=i,
                     HR=coxSummary$conf.int[,"exp(coef)"],
                     HR.95L=coxSummary$conf.int[,"lower .95"],
                     HR.95H=coxSummary$conf.int[,"upper .95"],
                     pvalue=coxSummary$coefficients[,"Pr(>|z|)"])
  )
}
write.table(uniTab,file="1-out_train_uniCox.txt",sep="\t",row.names=F,quote=F)
#write.csv(uniTab,file="indep_uniCox.csv",row.names=F,quote=F)


#多因素独立预后分析
rt=read.table("TCGA_KIRC_train_indep_unicox_input_treated.txt",header=T,sep="\t",check.names=F,row.names=1)
# uniTab$pvalue <- sapply(uniTab$pvalue,function(x){as.numeric(x)})
# uniTab[,c(2:ncol(uniTab))] <- apply(uniTab[,c(2:ncol(uniTab))],2,function(x){as.numeric(x)})
# 
# mod <- uniTab[uniTab$pvalue<=0.05,]$id   ##提取p<0.05的基因列表
# rt <- rt[,c("futime","fustat",mod)]
colnames(rt)
rt <- rt[,-c(3:6,8:10,14)] 


multiCox=coxph(Surv(futime, fustat) ~ ., data = rt)
multiCoxSum=summary(multiCox)
multiTab=data.frame()
multiTab=cbind(
  HR=multiCoxSum$conf.int[,"exp(coef)"],
  HR.95L=multiCoxSum$conf.int[,"lower .95"],
  HR.95H=multiCoxSum$conf.int[,"upper .95"],
  pvalue=multiCoxSum$coefficients[,"Pr(>|z|)"])
multiTab=cbind(id=row.names(multiTab),multiTab)
write.table(multiTab,file="2-out_train_multiCox.txt",sep="\t",row.names=F,quote=F)
#write.csv(multiTab,file="indep_multiCox.csv",row.names=F,quote=F)

multiTab <- data.frame(multiTab)
multiTab$pvalue <- sapply(multiTab$pvalue,function(x){as.numeric(x)})
multiTab[,c(2:ncol(multiTab))] <- apply(multiTab[,c(2:ncol(multiTab))],2,function(x){as.numeric(x)})


 mod1 <- multiTab[multiTab$pvalue<=0.05,]$id   ##提取p<0.05的基因列表
# rt1 <- rt[,c("futime","fustat",mod1)]
# write.csv(rt1,file="nomo_input.csv",row.names=T,quote=F)
# 



############绘制森林图函数############
bioForest=function(coxFile=null,color=null,forestFile=null){
  #读取输入文件
  rt <- read.table(coxFile,header=T,sep="\t",row.names=1,check.names=F)
  gene <- rownames(rt)
  hr <- sprintf("%.3f",rt$"HR")
  hrLow  <- sprintf("%.3f",rt$"HR.95L")
  hrHigh <- sprintf("%.3f",rt$"HR.95H")
  Hazard.ratio <- paste0(hr,"(",hrLow,"-",hrHigh,")")
  pVal <- ifelse(rt$pvalue<0.001, "<0.001", sprintf("%.3f", rt$pvalue))
  
  #输出图形
  pdf(file=forestFile, width = 7,height = 4)
  n <- nrow(rt)
  nRow <- n+1
  ylim <- c(1,nRow)
  layout(matrix(c(1,2),nc=2),width=c(3,2.5))
  
  #绘制森林图左边的临床信息
  xlim = c(0,3)
  par(mar=c(4,2.5,2,1))
  plot(1,xlim=xlim,ylim=ylim,type="n",axes=F,xlab="",ylab="")
  text.cex=0.7
  text(0,n:1,gene,adj=0,cex=text.cex)
  text(1.5-0.5*0.2,n:1,pVal,adj=1,cex=text.cex);text(1.5-0.5*0.2,n+1,'pvalue',cex=text.cex,font=2,adj=1)
  text(3,n:1,Hazard.ratio,adj=1,cex=text.cex);text(3,n+1,'Hazard ratio',cex=text.cex,font=2,adj=1,)
  
  #绘制森林图
  par(mar=c(4,1,2,1),mgp=c(2,0.5,0))
  xlim = c(0,max(as.numeric(hrLow),as.numeric(hrHigh)))  ###改HR的起始位置坐标
  plot(1,xlim=xlim,ylim=ylim,type="n",axes=F,ylab="",xaxs="i",xlab="Hazard ratio")
  arrows(as.numeric(hrLow),n:1,as.numeric(hrHigh),n:1,angle=90,code=3,length=0.05,col="darkblue",lwd=2.5)
  abline(v=1,col="black",lty=2,lwd=2)
  boxcolor = ifelse(as.numeric(hr) > 1, color, color)
  points(as.numeric(hr), n:1, pch = 15, col = boxcolor, cex=1.3)
  axis(1)
  dev.off()
}
############绘制森林图函数############

bioForest(coxFile="1-out_train_uniCox.txt",color="green",forestFile="uniForest.pdf")
bioForest(coxFile="2-out_train_multiCox.txt",color="red",forestFile="multiForest.pdf")




