#使用国内镜像安装包
options("repos"= c(CRAN="https://mirrors.tuna.tsinghua.edu.cn/CRAN/"))
options(BioC_mirror="http://mirrors.tuna.tsinghua.edu.cn/bioconductor/")

#加载包

library(ggplot2)
library(grid)
library(gridExtra)

Sys.setenv(LANGUAGE = "en") #显示英文报错信息
options(stringsAsFactors = FALSE) #禁止chr转成factor

#自定义函数，用于计算 Immunophenoscore(IPS)
ipsmap<- function (x) {
  if (x<=0) {
    ips<-0
  } else {
    if (x>=3) {
      ips<-10
    } else {
      ips<-round(x*10/3, digits=0)
    }
  }
  return(ips)
}

#分配颜色
my_palette <- colorRampPalette(c("blue", "white", "red"))(n = 1000)
mapcolors<-function (x) {
  za<-NULL
  if (x>=3) {
    za=1000
  } else {
    if (x<=-3) {
      za=1
    } else {
      za=round(166.5*x+500.5,digits=0)
    }
  }
  return(my_palette[za])
}
my_palette2 <- colorRampPalette(c("black", "white"))(n = 1000)
mapbw<-function (x) {
  za2<-NULL
  if (x>=2) {
    za2=1000
  } else {
    if (x<=-2) {
      za2=1
    } else {
      za2=round(249.75*x+500.5,digits=0)
    }
  }
  return(my_palette2[za2])
}


#输入文件

#easy_input_expr.csv，表达矩阵。例如log2(TPM+1)，每行一个基因，每列一个样本。
#easy_input_IPS_genes.txt，IPS基因权重文件。IPS相关基因、分类及其权重。

## 读取表达谱文件
load("C:\\Users\\Administrator\\Desktop\\243-11\\00-data\\01-TCGA_BRCA--FPKM\\03_TCGA_BRCA_uniq.symbol_1204.Rdata")
# data <- out[,-1]
data2[1:4,1:4]

#expr <- all

expr <- read.table("TCGA_BRCA_exp.txt",row.names=1,header=TRUE, sep="\t",check.names=FALSE,stringsAsFactors = FALSE)
expr[1:3,1:3]

inter <- intersect(colnames(data2),colnames(expr))
expr <- expr[,inter]

## 取出肿瘤样本
tumsam <- colnames(expr)[substr(colnames(expr),14,15) == "01"] # 取出肿瘤样本
#gene_expression <- log2(expr[,tumsam] + 1) # 对TPM值做对数转化
gene_expression <- expr[,tumsam] 
sample_names <- tumsam


# gene_expression <- expr
# sample_names <- colnames(expr)


## 修改几个基因名
## IPS基因权重文件中的基因名跟常用的gene symbol不同，我们把表达矩阵里的gene symbol修改成跟IPS基因列表一致的基因名。
if(is.element("CAVIN2",rownames(gene_expression))) {
  rownames(gene_expression) <- gsub("CAVIN2","SDPR",rownames(gene_expression)) # 根据https://www.genecards.org上的相关信息进行同名转换
}
# 另外注意示例数据的里的CCL3L3的注释源自GENCODE27，GeneCards显示CCL3L1的ENSEMBL为ENSG00000276085，该编码在GENCODE27里注释为CCL3L3，因此修改
if(is.element("CCL3L3",rownames(gene_expression))) {
  rownames(gene_expression) <- gsub("CCL3L3","CCL3L1",rownames(gene_expression))
}



## 读取IPS相关基因以及其权重
IPSG <- read.table("easy_input_IPS_genes.txt",header=TRUE, sep="\t",check.names=FALSE,stringsAsFactors = FALSE,row.names = NULL)
head(IPSG)

unique_ips_genes <- as.vector(unique(IPSG$NAME))

## 初始化数据
IPS <- MHC <- CP <- EC <- SC <- AZ <- NULL

# 获取表达谱里的基因名
GVEC <- row.names(gene_expression)

# 获取IPS基因文件里的基因名
VEC <- as.vector(IPSG$GENE)

# 匹配基因并找到缺失基因(正常情况MISSINGZ_GENES应该是empty)
ind <- which(is.na(match(VEC,GVEC)))
MISSING_GENES <- VEC[ind]
head(MISSING_GENES,20)


# a <- expr[rownames(expr) %in% "HLA-A",]


dat <- IPSG[ind,]
if (length(MISSING_GENES) > 0) { # 若存在缺失基因报出（如果上面两个基因不修改，这里是要报缺失的，会导致最终结果错误）
  message(paste0("--differently named or missing genes: ",paste(MISSING_GENES,collapse = ", ")))
  print(IPSG[ind,])
  message("please check data and make sure all genes matches!")
} else {
  message("--all genes matched!") # 请确保所有基因都匹配!!!
}

#计算并画图
picDir="picture"                                               #????ͼƬ????Ŀ¼??????
dir.create(picDir)

## 循环每个样本
for (i in 1:length(sample_names)) { 
  GE <- gene_expression[[i]]
  mGE <- mean(GE)
  sGE <- sd(GE)
  Z1 <- (gene_expression[as.vector(IPSG$GENE),i]-mGE)/sGE
  W1 <- IPSG$WEIGHT
  WEIGHT <- MIG <- NULL
  k <- 1
  for (gen in unique_ips_genes) {
    MIG[k] <- mean(Z1[which (as.vector(IPSG$NAME)==gen)],na.rm=TRUE)
    WEIGHT[k] <- mean(W1[which (as.vector(IPSG$NAME)==gen)])
    k <- k + 1
  }
  WG <- MIG * WEIGHT
  MHC[i] <- mean(WG[1:10])
  CP[i] <- mean(WG[11:20])
  EC[i] <- mean(WG[21:24])
  SC[i] <- mean(WG[25:26])
  AZ[i] <- sum(MHC[i],CP[i],EC[i],SC[i])
  IPS[i] <- ipsmap(AZ[i])
  
  # 绘制Immunophenogram
  data_a <- data.frame (start = c(0,2.5,5,7.5,10,15,seq(20,39),0,10,20,30), end = c(2.5,5,7.5,10,15,seq(20,40),10,20,30,40), y1=c(rep(2.6,26),rep(0.4,4)),y2=c(rep(5.6,26),rep(2.2,4)),z=c(MIG[c(21:26,11:20,1:10)],EC[i],SC[i],CP[i],MHC[i]),vcol=c(unlist(lapply(MIG[c(21:26,11:20,1:10)],mapcolors)), unlist(lapply(c(EC[i],SC[i],CP[i],MHC[i]),mapbw))), label = c(unique_ips_genes[c(21:26,11:20,1:10)],"EC","SC","CP","MHC"))
  data_a$label <- factor(data_a$label, levels=unique(data_a$label))
  plot_a1 <- ggplot() + 
    geom_rect(data=data_a, mapping=aes(xmin=start, xmax=end, ymin=y1, ymax=y2, fill=label), size=0.5,color="black", alpha=1) +
    coord_polar()  + 
    scale_y_continuous(limits = c(0, 6)) +
    scale_fill_manual(values =as.vector(data_a$vcol),guide=FALSE) +
    theme_bw() + 
    theme(panel.spacing = unit(0, 'mm'), 
          panel.grid.major = element_blank(),
          panel.grid.minor = element_blank(),
          panel.border = element_blank(),
          panel.background = element_blank(),
          axis.line = element_line(colour = "white"),
          axis.text=element_blank(), 
          axis.ticks= element_blank()) + 
    geom_text(aes(x=5, y=1.3, label="EC"), size=4) +
    geom_text(aes(x=15, y=1.3, label="SC"), size=4) + 
    geom_text(aes(x=25, y=1.3, label="CP"), size=4) + 
    geom_text(aes(x=35, y=1.3, label="MHC"), size=4)
  plot_a2 <- plot_a1 +
    geom_text(aes(x=1.25, y=4.1, label="+ Act CD4"), angle=78.75, size=4) +
    geom_text(aes(x=3.75, y=4.1, label="+ Act CD8"),angle=56.25, size=4) +
    geom_text(aes(x=6.25, y=4.1, label="+ Tem CD4"), angle=33.75,size=4) +
    geom_text(aes(x=8.75, y=4.1, label="+ Tem CD8"), angle=11.25,size=4) +
    geom_text(aes(x=17.5, y=4.1, label="- MDSC"), angle=-67.5,size=4) +
    geom_text(aes(x=12.5, y=4.1, label="- Treg"), angle=-22.5,size=4)
  plot_a3 <- plot_a2 +
    geom_text(aes(x=20.5, y=4.1, label="PD-1 -"), angle=85.5, size=4) +
    geom_text(aes(x=21.5, y=4.1, label="CTLA4 -"), angle=76.5, size=4) +
    geom_text(aes(x=22.5, y=4.1, label="LAG3 -"), angle=67.5, size=4) +
    geom_text(aes(x=23.5, y=4.1, label="TIGIT -"), angle=58.5, size=4) +
    geom_text(aes(x=24.5, y=4.1, label="TIM3 -"), angle=49.5, size=4) +
    geom_text(aes(x=25.5, y=4.1, label="PD-L1 -"), angle=40.5, size=4) +
    geom_text(aes(x=26.5, y=4.1, label="PD-L2 -"), angle=31.5, size=4) +
    geom_text(aes(x=27.5, y=4.1, label="CD27 +"), angle=22.5, size=4) +
    geom_text(aes(x=28.5, y=4.1, label="ICOS +"), angle=13.5, size=4) +
    geom_text(aes(x=29.5, y=4.1, label="IDO1 -"), angle=4.5, size=4)
  plot_a4 <- plot_a3 +
    geom_text(aes(x=30.5, y=4.1, label="B2M +"), angle=-4.5, size=4) +
    geom_text(aes(x=31.5, y=4.1, label="TAP1 +"), angle=-13.5, size=4) +
    geom_text(aes(x=32.5, y=4.1, label="TAP2 +"), angle=-22.5, size=4) +
    geom_text(aes(x=33.5, y=4.1, label="HLA-A +"), angle=-31.5, size=4) +
    geom_text(aes(x=34.5, y=4.1, label="HLA-B +"), angle=-40.5, size=4) +
    geom_text(aes(x=35.5, y=4.1, label="HLA-C +"), angle=-49.5, size=4) +
    geom_text(aes(x=36.5, y=4.1, label="HLA-DPA1 +"), angle=-58.5, size=4) +
    geom_text(aes(x=37.5, y=4.1, label="HLA-DPB1 +"), angle=-67.5, size=4) +
    geom_text(aes(x=38.5, y=4.1, label="HLA-E +"), angle=-76.5, size=4) +
    geom_text(aes(x=39.5, y=4.1, label="HLA-F +"), angle=-85.5, size=4)
  plot_a5 <- plot_a4 +
    geom_text(aes(x=0, y=6, label=paste("Immunophenoscore: ",IPS[i],sep="")), angle=0,size=6,vjust=-0.5) + 
    theme(axis.title=element_blank())
  plot_a <- plot_a5 + theme(plot.margin=unit(c(0,0,0,0),"mm")) +
    geom_text(vjust=1.15,hjust=0,aes(x=25.5, y=6,label="\n\n\n\n   MHC: Antigen Processing                                 EC: Effector Cells\n   CP: Checkpoints | Immunomodulators              SC: Suppressor Cells\n\n", hjust = 0), size=4)
  
  ## 绘制图例 sample-wise (averaged) z-scores
  data_b <- data.frame (start = rep(0,23), 
                        end = rep(0.7,23), 
                        y1=seq(0,22,by=1), 
                        y2=seq(1,23,by=1),
                        z=seq(-3,3,by=6/22),
                        vcol=c(unlist(lapply(seq(-3,3,by=6/22),mapcolors))), 
                        label = LETTERS[1:23])
  data_b_ticks <- data.frame(x = rep(1.2, 7), 
                             value = seq(-3,3, by=1), 
                             y = seq(0,6, by=1)*(22/6) +0.5)
  legendtheme <- theme(plot.margin = unit(c(2,0,2,0),"inch"), 
                       panel.spacing = unit(0,"null"), 
                       panel.grid.major = element_blank(),
                       panel.grid.minor = element_blank(),
                       panel.border = element_blank(),
                       panel.background = element_blank(), 
                       axis.line = element_line(colour = "white"), 
                       axis.text=element_blank(), 
                       axis.ticks= element_blank(), 
                       axis.title.x=element_blank())
  plot_b <- ggplot(hjust=0) + 
    geom_rect(data=data_b, 
              mapping=aes(xmin=start, xmax=end, ymin=y1, ymax=y2, fill=label), size=0.5,color="black", alpha=1) +
    scale_x_continuous(limits = c(0, 1.5),expand = c(0,0)) +
    scale_fill_manual(values =as.vector(data_b$vcol),guide=FALSE) +
    geom_text(data=data_b_ticks, aes(x=x, y=y, label=value),hjust="inward", size=4) +
    theme_bw() +
    legendtheme + 
    ylab("Sample-wise (averaged) z-score")
  
  ## 绘制图例 weighted z-scores
  data_c <- data.frame(start = rep(0,23),
                       end = rep(0.7,23),
                       y1=seq(0,22,by=1),
                       y2=seq(1,23,by=1),
                       z=seq(-2,2,by=4/22),
                       vcol=c(unlist(lapply(seq(-2,2,by=4/22),mapbw))), 
                       label = LETTERS[1:23])
  data_c_ticks <- data.frame(x = rep(1.2, 5), 
                             value = seq(-2,2, by=1),
                             y = seq(0,4, by=1)*(22/4) +0.5)
  plot_c <- ggplot() + 
    geom_rect(data=data_c, 
              mapping=aes(xmin=start, xmax=end, ymin=y1, ymax=y2, fill=label),size=0.5,color="black", alpha=1) + 
    scale_x_continuous(limits = c(0, 1.5),expand = c(0,0)) + 
    scale_fill_manual(values =as.vector(data_c$vcol),guide=FALSE) + 
    geom_text(data=data_c_ticks, aes(x=x, y=y, label=value),hjust="inward", size=4) +
    theme_bw() + 
    legendtheme + 
    ylab("Weighted z-score")
  
  ## 每一个样本保存图片
  file_name<-paste(picDir,"\\","IPS_",sample_names[i],".pdf",sep="")
  pdf(file_name, width=10, height=8)
  grid.arrange(plot_a,plot_b,plot_c, ncol=3, widths=c(0.8,0.1,0.1))
  invisible(dev.off())
  
  message(paste0("--analysis of ", tumsam[i]," done..."))
}

# 构建结果，包括各项得分以及IPS
DF <- data.frame(SAMPLE=sample_names,
                 MHC=MHC,
                 EC=EC,
                 SC=SC,
                 CP=CP,
                 AZ=AZ,
                 IPS=IPS,
                 stringsAsFactors = F)

# 输出IPS到文件
write.table(DF,file = "output_IPS.txt", row.names = FALSE, col.names = TRUE, quote=FALSE, sep="\t") 

save(DF,file="IPS.Rdata")
load("IPS.Rdata")


######  加入高低打分组 ####################
DF <- read.table("output_IPS.txt",sep="\t",header=T,check.names=F) 
colnames(DF)
#rt <- read.table("risk.txt",sep="\t",row.names=1,header=T,check.names=F) 
load("06_CuSig_cutpoint.Rdata")
colnames(res.cat)

Cu_subtype <- merge(DF,res.cat[,c(1,5)],by.x="SAMPLE",by.y="id",all=F)
rownames(Cu_subtype) <- Cu_subtype$SAMPLE
Cu_subtype1 <- Cu_subtype[,-1]
save(Cu_subtype1,file="High_Low_score-IPS.Rdata")



library(ggplot2)
library(ggpubr)
library(magrittr)
library(ggsignif)
library(RColorBrewer)

load("High_Low_score-IPS.Rdata")
Cu_subtype1[1:4,1:4]
merge <- Cu_subtype1[order(Cu_subtype1$group),]
merge$group

colnames(merge)
p1=ggboxplot(merge,
             x='group',          #选择X轴用哪列数据
             y='CP',      #选择Y轴用哪列数据
             color='group',       #颜色根据哪一列决定
             bxp.errorbar=T,     #是否添加误差bar，一般添加美观
             bxp.errorbar.width=0.2,  
             palette="Set1",      #颜色风格
             #palette = "jama",
             #add = "point",
             add='jitter')+         #是否添加箱线图上的点点
  stat_compare_means(label ="p.format",label.x = 1.5)
#comparisons = my_comparisons,
#label = "p.signif":显示显著性 
#method = "t.test"  ：显示用的检验方法，默认wilcox.test法      
p1
##########  2.添加标题  #################################################

p2=p1+
  labs(title="",   #添加主标题
       #subtitle="xxx",    #添加次标题
       #caption="Data source:xx cancer",  #添加脚注
       x='',y='CP'            #x和y轴的名字
  )
p2

############   4.设置主题   #############################################
p3=p2+
  theme(
    plot.title=element_text(color='black',size=12,hjust=0.5),
    plot.subtitle=element_text(color='black',size=12,hjust=0.5),
    plot.caption=element_text(color='black',size=12,face='italic',hjust=1),
    axis.text.x=element_text(color='black',size=12,angle=0),  
    axis.text.y=element_text(color='black',size=12,angle=0),
    axis.title.x=element_text(color='black',size=12,angle=0),
    axis.title.y=element_text(color='black',size=12,angle=90),   
    #lengend.title=element_text(color='black',size=16),
    #lengend.text=element_text(color='black',size=16),
    # axis.line.y=element_line(color='black',linetype='solid'),
    # axis.line.x=element_line(color='black',linetype='solid'),
    # panel.border=element_rect(linetype='solid',size=1.2,fill=NA) #图四周框起来
  )
#p3        
pdf(file="CP.pdf",width=6,height=6)
p3 
dev.off()









#开始画图
#用ggplot2画图，用ggpubr算p value。
library(ggpubr)
library(RColorBrewer)
display.brewer.all()  #显示所有可用颜色
#mycol <- brewer.pal(6,'Set1')

mycol <- c("darkgreen", "darkorchid3", "orange") #与分组数量一致

###1.简单版箱线图  ##########
p <- ggplot(merge, aes(x = risk, y = IPS, color = risk)) +
  geom_boxplot(outlier.color = NA) + #隐去箱线图上的异常点
  scale_color_manual(values = mycol) + #自定义配色
  stat_compare_means(#paired = T, #whether you want a paired test
    #两组对比，用wilcox.test或t.test
    method = "wilcox.test", 
    #多组对比，用kruskal.test或anova
    #method = "kruskal.test",
    label.y = max(merge$IPS)*1.1) + #label的位置
  theme_bw() + #去除背景色
  theme(panel.grid =element_blank()) + #去除网格线
  theme(panel.border = element_blank()) + #去除外层边框
  theme(axis.line = element_line(colour = "black")) + #沿坐标轴显示直线
  xlab("risk") + 
  #ylim(0,5) + #设置Y轴范围
  guides(color=FALSE) #不显示图例
p

ggsave("IPS_risk.pdf")
# 2，绘制Wikinson点图
p + geom_dotplot(binaxis = "y", #沿y轴堆积，并沿着x轴分组
                 binwidth = 0.5, #最大组距
                 dotsize = 1, #点的大小
                 #如果点太多，两组叠在一起，就需要运行下面这行把它们分开
                 #stackgroups = T, binpositions="all",
                 stackdir = "center")  #数量保持一致的中心堆叠方式

# 3,或者散点图
pdf("dot_IPS_risk.pdf",height=6,w=6)
p + geom_point(aes(group = risk),
               alpha=.3, #点太多，设为透明色，就能看到叠加效果
               size = 2, #点的大小
               position="jitter") #分散
dev.off()


################### 好几个评分的箱线图  ######################
library(tidyverse)
library(tidyr)
library(ggplot2)
require(ggpubr)
require(ggsci)
require(cowplot)

library(RColorBrewer)
load("risk-IPS.Rdata")

hlaexp<-merge[,-1]
hlaexp <- hlaexp[,-c(ncol(hlaexp)-1,ncol(hlaexp))]
boxdata <- gather(hlaexp,HLA,exp,2:ncol(hlaexp))
boxdata[,'exp'] <- sapply(boxdata[,'exp'],function(x){log2(x+1)})

#group分组列变成factor,boxplot按照L,H排列
boxdata$group <- factor(boxdata$risk,levels = c('Low_risk','High_risk'))

display.brewer.all()  #显示所有可用颜色
mycol <- brewer.pal(2,'Dark2')
pdf('MHC-EC-SC-CP.pdf',width =7,height = 5)
ggboxplot(boxdata,x='HLA',y='exp',color ='group',
          ylab = 'Relative Expression',
          xlab ='',palette = mycol,  #nejm #jco #jama#npg#d3#aaas,"aaas"
          add ='jitter',add.params = list(size=1,shape=16),   #add ='jitter'
          width=0.6)+
  #rotate_x_text(45)+
  #coord_flip()+   #颠倒X轴和Y轴
  theme(axis.text=element_text(size=8),
        axis.title = element_text(size=10),
        legend.text = element_text(size=8),
        legend.title = element_text(size=10))+
  stat_compare_means(size = 3,aes(group=group),
                     label = 'p.signif',method = 'wilcox.test',hide.ns = F)  ##检验方法可以改

dev.off()
