
###################所需软件的安装 ###############################################     
install.packages('BiocManager') 
BiocManager::install("stringr")
BiocManager::install("clusterProfiler")
BiocManager::install("org.Hs.eg.db")
BiocManager::install("GOplot")

library(clusterProfiler)
library(org.Hs.eg.db)
library(GOplot)
library(stringr)

###########准备第一个输入文件,内容包括ID,term,Genes,adj_pval,category######################################
#Hubgene <- read.csv("jVenn.csv",header=T,)
#gene_diff <- Hubgene$Degree.Closeness.Stress.MNC.Radiality

Hubgene <- read.table("diffSig_logFC0.5_p0.05_590.xls",header=T,sep="\t")
gene_diff <- Hubgene$gene
head(gene_diff)  

#使用bitr函数进行id转换，使用的是bioconductor的db系列包
#eg <- bitr(gene_diff, fromType="SYMBOL", toType=c("ENTREZID","ENSEMBL",'SYMBOL'), OrgDb="org.Hs.eg.db")
#head(eg)

##富集分析   ####
ego <- enrichGO(gene_diff,   ##用基因名或是ENTREZID进行GO富集分析(相应的来改gene_diff)
                keyType = "SYMBOL",
                OrgDb= org.Hs.eg.db,  # OrgDb:物种注释数据
                ont = "ALL",          # ont:指三个层面来阐述基因功能,生物学过程(BP),细胞组分(CC),分子功能（MF）
                pAdjustMethod = "BH",
                minGSSize = 1,
                pvalueCutoff = 0.05,
                qvalueCutoff = 0.05)
class(ego)
#3.转换成图的输入数据
ego <- data.frame(ego)  ##将GO富集结果转换成数据框格式
write.csv(ego,"cluster_go.csv")  ##读出富集结果
colnames(ego)  ##查看第一行，即表头
ego <- ego[1:10,c(1,2,3,9,7)]    ##保留第1,2,3,9,7列数据，即ONTOLOGY,ID,Description,geneID,p.adjust数据。
ego$geneID <- str_replace_all(ego$geneID,"/",",")   ##替换geneID列中所有的“/”为“,”。
names(ego)=c("Category","ID","Term","Genes","adj_pval")   ##四列数据命名为ID,Term,Genes,adj_pval。
#ego$Category = "BP"    ##加一列数据，表头是Category，格内全为BP。
head(ego) 
##到此，准备好第一个输入文件(ego)了,内容包括ID,term,Genes,adj_pval,category


##################准备第二个文件，内容包括ID（即基因）和logFC######################
#genes_ <- read.csv("brown_gene_上下调信息.csv",header=T)
genes <- Hubgene[,c(1,2)]  #保留gene和logFC列
colnames(genes)[1] <- "ID"
#genes = data.frame(ID=deg$symbol,logFC=deg$logFC)  #制作文件，ID和logFC两列数据
head(genes)

######## 出图 #########################################

circ <- circle_dat(ego,genes);   #用ego和genes俩个文件
#zscore: 每个GO term下上调(logFC>0)基因数和下调基因数的差与注释到GO term基因数平方根的商
head(circ)
pdf(file="GOcircle.pdf",width=12,height=10,onefile=FALSE)
GOCircle(circ)  #绘图
dev.off()



###############----------------------------KEGG--------------------------------
##富集分析   ####
kegg <- enrichKEGG(gene = gene_diff, organism = "hsa", pvalueCutoff =0.05, qvalueCutoff =0.05)
write.table(kegg,file="KEGG.txt",sep="\t",quote=F,row.names = F)
class(ego)


#3.转换成图的输入数据
ego <- data.frame(ego)  ##将GO富集结果转换成数据框格式
write.csv(ego,"cluster_go.csv")  ##读出富集结果
colnames(ego)  ##查看第一行，即表头
ego <- ego[1:10,c(1,2,3,9,7)]    ##保留第1,2,3,9,7列数据，即ONTOLOGY,ID,Description,geneID,p.adjust数据。
ego$geneID <- str_replace_all(ego$geneID,"/",",")   ##替换geneID列中所有的“/”为“,”。
names(ego)=c("Category","ID","Term","Genes","adj_pval")   ##四列数据命名为ID,Term,Genes,adj_pval。
#ego$Category = "BP"    ##加一列数据，表头是Category，格内全为BP。
head(ego) 
##到此，准备好第一个输入文件(ego)了,内容包括ID,term,Genes,adj_pval,category

######## 出图 #########################################

circ <- circle_dat(ego,genes);   #用ego和genes俩个文件
#zscore: 每个GO term下上调(logFC>0)基因数和下调基因数的差与注释到GO term基因数平方根的商
head(circ)
pdf(file="GOcircle.pdf",width=12,height=10,onefile=FALSE)
GOCircle(circ)  #绘图
dev.off()




GOCircle(circ, nsub = c('GO:0007507','GO:0008092', 'GO:0008047'))
#根据GO Term选择要展示的GO Term
GOCircle(circ, nsub = 10)
#选择要展示的GO Term的数量

###########GO气泡图###################
GOBubble(circ, labels = 3)
##########GO同时展示BP, CC, MF的气泡图#################
GOBubble(circ, title = 'Bubble plot', colour = c('orange', 'darkred', 'gold'), display = 'multiple', labels = 3)  
############展示基因和GO Terms关系的圈图（GOChord）###########


